<?php

namespace Drupal\configuration\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

class ConfigurationForm extends ConfigFormBase
{
  /**
   * @var AccountInterface
   */
  protected AccountInterface $currentUser;

  public function __construct(ConfigFactoryInterface $config_factory, AccountInterface $current_user)
  {
    parent::__construct($config_factory);

    $this->currentUser = $current_user;
  }

  public static function create(ContainerInterface $container)
  {
    return new static(
      $container->get('config.factory'),
      $container->get('current_user')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array
  {
    return ['configuration.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string
  {
    return 'configuration_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array
  {
    $form = parent::buildForm($form, $form_state);
    $config = $this->config('configuration.settings');

    $is_local = str_contains($GLOBALS['base_url'], 'localhost');

    $default_public_api_url = $is_local ? 'http://nexora-nc-drupal-1:80/' : $config->get('public_api_url');

    $form['client_secret'] = [
      '#type' => 'textfield',
      '#title' => t('Client secret'),
      '#default_value' => $config->get('client_secret'),
      '#required' => TRUE,
    ];

    $form['client_id'] = [
      '#type' => 'textfield',
      '#title' => t('Client ID'),
      '#default_value' => $config->get('client_id'),
      '#required' => TRUE,
    ];

    $form['public_api_url'] = [
      '#type' => 'textfield',
      '#title' => t('Public API URL'),
      '#default_value' => $default_public_api_url,
      '#required' => TRUE,
    ];

    $form['email_subject'] = [
      '#type' => 'textfield',
      '#title' => t('Email subject'),
      '#default_value' => $config->get('email_subject'),
      '#required' => TRUE,
    ];

    $form['email_body'] = [
      '#type' => 'textarea',
      '#title' => t('Email body'),
      '#default_value' => $config->get('email_body'),
      '#required' => TRUE,
      '#description' => $this->t(
        'Use placeholder @new_password for User new password.'
      ),
    ];

    $form['cosmos_api_url'] = [
      '#type' => 'textfield',
      '#title' => t('Cosmos API URL'),
      '#default_value' => $config->get('cosmos_api_url'),
      '#required' => TRUE,
    ];

    $form['cosmos_api_client_id'] = [
      '#type' => 'textfield',
      '#title' => t('Cosmos API Client ID'),
      '#default_value' => $config->get('cosmos_api_client_id'),
      '#required' => TRUE,
    ];

    $form['cosmos_api_client_secret'] = [
      '#type' => 'textfield',
      '#title' => t('Cosmos API Client Secret'),
      '#default_value' => $config->get('cosmos_api_client_secret'),
      '#required' => TRUE,
    ];

    $form['cosmos_api_refresh_token'] = [
      '#type' => 'textfield',
      '#title' => t('Cosmos API refresh token'),
      '#default_value' => $config->get('cosmos_api_refresh_token'),
      '#required' => TRUE,
    ];

    $form['cosmos_api_access_token'] = [
      '#type' => 'textfield',
      '#title' => t('Cosmos API access token'),
      '#default_value' => $config->get('cosmos_api_access_token'),
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc }
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void
  {
    if ($this->currentUser->hasPermission('administer site configuration')) {
      $config = $this->config('configuration.settings');
      $config->set('client_secret', $form_state->getValue('client_secret'));
      $config->set('client_id', $form_state->getValue('client_id'));
      $config->set('public_api_url', $form_state->getValue('public_api_url'));
      $config->set('email_subject', $form_state->getValue('email_subject'));
      $config->set('email_body', $form_state->getValue('email_body'));
      $config->set('cosmos_api_url', $form_state->getValue('cosmos_api_url'));
      $config->set('cosmos_api_client_id', $form_state->getValue('cosmos_api_client_id'));
      $config->set('cosmos_api_client_secret', $form_state->getValue('cosmos_api_client_secret'));
      $config->set('cosmos_api_refresh_token', $form_state->getValue('cosmos_api_refresh_token'));
      $config->set('cosmos_api_access_token', $form_state->getValue('cosmos_api_access_token'));
      $config->save();
      parent::submitForm($form, $form_state);
    }
  }
}
