<?php

namespace Drupal\cosmos_rest_api\Cron;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\PluginBase;
use GuzzleHttp\Exception\RequestException;
use Symfony\Component\DependencyInjection\ContainerInterface;
use GuzzleHttp\ClientInterface;
use Psr\Log\LoggerInterface;

/**
 * Provides a 'MyCronClass' cron plugin.
 *
 * @Cron(
 *   id = "cosmos_rest_api_cron_service",
 *   label = @Translation("Cosmos Rest API Cron Service"),
 *   weight = 1
 * )
 */
class CosmosCronService extends PluginBase implements ContainerFactoryPluginInterface
{

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $client;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a new MyCronClass object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param array $serializer_formats
   *   The available serialization formats.
   * @param \Psr\Log\LoggerInterface $logger
   *   A logger instance.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \GuzzleHttp\ClientInterface $client
   *   The HTTP client.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory.
   */
  public function __construct(
    EntityTypeManagerInterface $entityTypeManager,
    ClientInterface            $client,
    ConfigFactoryInterface     $configFactory
  )
  {
    $this->entityTypeManager = $entityTypeManager;
    $this->client = $client;
    $this->configFactory = $configFactory;
    parent::__construct([], 'cosmos_cron_service', []);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array              $configuration,
                       $plugin_id,
                       $plugin_definition
  )
  {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('http_client'),
      $container->get('config.factory')
    );
  }


  /**
   * Execute the cron task.
   */
  public function executeCronTask()
  {
    try {
      // Get access token using the refresh token.
//      $accessToken = $this->getAccessToken();
      $config = $this->configFactory->getEditable('configuration.settings');
      $accessToken = $config->get('cosmos_api_access_token');

      // Use the access token to make an API request.
      $data = $this->getFacilities($accessToken);


    } catch (RequestException $e) {
      // Handle request exception.
      // You may want to log errors or implement custom error handling.
    }
  }

  /**
   * Get access token using the refresh token.
   *
   * @return string
   *   The access token.
   */
  protected function getAccessToken()
  {

    $config = $this->configFactory->getEditable('configuration.settings');
    $cosmos_secret = $config->get('cosmos_api_client_secret');
    $cosmos_client_id = $config->get('cosmos_api_client_id');
    $cosmos_refresh_token = $config->get('cosmos_api_refresh_token');
    $cosmos_api_url = $config->get('cosmos_api_url');

    $response = $this->client->post($cosmos_api_url . '/oauth/token', [
      'form_params' => [
        'grant_type' => 'refresh_token',
        'client_id' => $cosmos_client_id,
        'client_secret' => $cosmos_secret,
        'refresh_token' => $cosmos_refresh_token,
        'scopes' => 'public'
      ],
    ]);
    $data = json_decode($response->getBody(), true);
    $config->set('cosmos_api_refresh_token', $data['refresh_token']);
    $config->save();

    return $data['access_token'];
  }

  protected function setChargers($faciltyNode, $chargers, $clientId = null)
  {
    $sockets = '';
    foreach ($chargers as $charger) {
      if (array_key_exists('sockets', $charger)) {
        $socketData = json_encode($charger['sockets']);
        $socketDecoded = json_decode($socketData, JSON_UNESCAPED_UNICODE);
        $sockets = json_encode($socketDecoded);
      }
      $storage = $this->entityTypeManager->getStorage('node');
      $query = $storage->getQuery()
        ->condition('title', $charger['charge_point_name'])
        ->condition('type', 'charger')
        ->accessCheck(TRUE)
        ->range(0, 1); // Limit to one result, assuming you expect only one match.
      $entity_ids = $query->execute();
      if (!empty($entity_ids)) {
        $entity_id = reset($entity_ids);
        $entity = $storage->load($entity_id);
        if (empty($entity->get('field_charger_id')->getValue()) || empty($entity->get('field_socket')->getValue())) {
          $entity->set('field_charger_id', $charger['charge_point_id']);
          $entity->set('field_latitude', $charger['charge_point_latitude']);
          $entity->set('field_longitude', $charger['charge_point_longitude']);
          $entity->set('field_socket', $sockets);
          $entity->save();
        }
      } else {

        $node_data = [
          'type' => 'charger',
          'title' => $charger['charge_point_name'],
          'field_charger_id' => $charger['charge_point_id'],
          'field_facility' => $faciltyNode->id(),
          'uid' => $clientId,
          'field_latitude' => $charger['charge_point_latitude'],
          'field_longitude' => $charger['charge_point_longitude'],
          'field_socket' => $sockets
        ];

        $node = $this->entityTypeManager->getStorage('node')
          ->create($node_data);
        $node->save();
      }
    }
  }

  protected function setFacilityNodes($data)
  {
    foreach ($data['facilities'] as $facility) {
      $storage = $this->entityTypeManager->getStorage('node');
      $query = $storage->getQuery()
        ->condition('title', $facility['facility_name'])
        ->condition('type', 'facility')
        ->accessCheck(TRUE)
        ->range(0, 1); // Limit to one result, assuming you expect only one match.
      $entity_ids = $query->execute();
      if (!empty($entity_ids)) {
        $entity_id = reset($entity_ids);
        $entity = $storage->load($entity_id);
        if (empty($entity->get('field_facility_id')->getValue())) {
          $entity->set('field_facility_id', $facility['facility_id']);
          $entity->save();
        }
        $this->setChargers($entity, $facility['charge_points'], $entity->get('field_client')->getValue()[0]['target_id']);
      } else {
        $city = explode(' ', trim($facility['facility_address']));
        $address = [
          'country_code' => 'MK',
          'address_line1' => strstr($facility['facility_address'], '-', true) !== false ? $facility['facility_address'] : 'Hyphen not found',
          'locality' => end($city),
        ];

        $user_storage = $this->entityTypeManager->getStorage('user');
        /** @var \Drupal\user\UserInterface $client */
        $clients = $user_storage->loadByProperties(['name' => strtok($facility['facility_name'], " ")]);
        $client = reset($clients);
        $node_data = [
          'type' => 'facility',
          'title' => $facility['facility_name'],
          'field_name' => $facility['facility_name'],
          'field_facility_id' => $facility['facility_id'],
          'field_address' => $address,
          'field_client' => $client->id(),
          'uid' => $client->id()
        ];

        $node = $this->entityTypeManager->getStorage('node')
          ->create($node_data);
        $node->save();
        $this->setChargers($node, $facility['charge_points'], $client->id());
      }
    }
  }


  /**
   * Get data from the API using the access token.
   *
   * @param string $accessToken
   *   The access token.
   *
   * @return array
   *   The Facilities response data.
   */
  protected function getFacilities($accessToken)
  {
    $cosmos_api_url = $this->configFactory->get('configuration.settings')->get('cosmos_api_url');
    $requestUrl = $cosmos_api_url . '/api/v3/charge_points';
    $response = $this->client->get($requestUrl, [
      'headers' => [
        'Authorization' => 'Bearer ' . $accessToken,
      ],
    ]);

    $data = json_decode($response->getBody(), true);
    $this->setFacilityNodes($data);

  }
}
