<?php

namespace Drupal\user_rest_api\Plugin\rest\resource;

use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ResourceResponse;
use GuzzleHttp\Exception\ClientException;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use GuzzleHttp\Client;
use Nowakowskir\JWT\JWT;
use Nowakowskir\JWT\TokenDecoded;
use Nowakowskir\JWT\TokenEncoded;

/**
 * Provides Social Login Resource
 *
 * @RestResource (
 *   id = "social_login_resource",
 *   label = @Translation ("Social Login Resource"),
 *   uri_paths = {
 *    "create" = "api/v1/social-login"
 *   }
 * )
 */
class SocialLoginResource extends ResourceBase
{
  /** @var EntityTypeManagerInterface */
  protected EntityTypeManagerInterface $entityTypeManager;

  /** @var Client */
  protected Client $client;

  /** @var ConfigFactoryInterface */
  protected ConfigFactoryInterface $configFactory;

  public function __construct(
    array                      $configuration,
                               $plugin_id,
                               $plugin_definition,
    array                      $serializer_formats,
    LoggerInterface            $logger,
    EntityTypeManagerInterface $entity_type_manager,
    Client                     $client,
    ConfigFactoryInterface     $config_factory,
  )
  {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);
    $this->entityTypeManager = $entity_type_manager;
    $this->client = $client;
    $this->configFactory = $config_factory;
  }

  public static function create(
    ContainerInterface $container,
    array              $configuration,
                       $plugin_id,
                       $plugin_definition,
  )
  {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->getParameter('serializer.formats'),
      $container->get('logger.factory')->get('sample_rest_resource'),
      $container->get('entity_type.manager'),
      $container->get('http_client'),
      $container->get('config.factory'),
    );
  }

  /**
   * @throws InvalidPluginDefinitionException
   * @throws PluginNotFoundException
   */
  public function post($data): ResourceResponse
  {
    $tokenEncoded = new TokenEncoded($data['token']);
    $payload = $tokenEncoded->decode()->getPayload();
    $email = $payload['email'];
    $full_name = $payload['name'];

    $users = $this->entityTypeManager->getStorage('user')
      ->loadByProperties(['mail' => $email]);
    $user = reset($users);

    if (!$user) {
      $new_user = $this->createUser($email, $full_name);
      $token = $this->getToken($new_user);

      return new ResourceResponse([
        'status' => 'Success',
        'message' => 'Successful user created.',
        'data' => [
          'id' => $new_user->id(),
          'name' => $new_user->field_customer_name[0]->value,
          'surname' => $new_user->field_customer_surname[0]->value,
          'email' => $new_user->mail[0]->value,
          'token_type' => $token['token_type'],
          'expires_in' => $token['expires_in'],
          'access_token' => $token['access_token'],
          'refresh_token' => $token['refresh_token']
        ]
      ]);
    } else {
      $token = $this->loginUser($user);

      return new ResourceResponse([
        'status' => 'Success',
        'message' => 'Successful user logged in.',
        'data' => [
          'id' => $user->id(),
          'token_type' => $token['token_type'],
          'expires_in' => $token['expires_in'],
          'access_token' => $token['access_token'],
          'refresh_token' => $token['refresh_token']
        ]
      ]);
    }
  }

  public function createUser($email, $full_name)
  {
    $user = $this->entityTypeManager->getStorage('user')->create();
    $nameParts = explode(' ', $full_name);
    $name = $nameParts[0];
    $surname = $nameParts[1];
    $password = base64_encode($email);

    $user->setEmail($email);
    $user->setPassword($password);
    $user->addRole('customer');
    $user->set('preferred_admin_langcode', 'en');
    $user->set('init', $email);
    $user->set('field_customer_name', $name);
    $user->set('field_customer_surname', $surname);
    $user->set('field_is_social', TRUE);
    $user->activate();

    $user->save();

    return $user;
  }

  public function loginUser($user)
  {
    try {
      $account = \Drupal\user\Entity\User::load($user->id());

      $username = $account->get('name')[0]->getValue();
      $generate_username = $username['value'];
      $password = base64_encode($user->getEmail());

      $config = $this->configFactory->get('configuration.settings');
      $client_secret = $config->get('client_secret');
      $client_id = $config->get('client_id');
      $public_api_url = $config->get('public_api_url');

      $response = $this->client->post($public_api_url . 'oauth/token', [
        'form_params' => [
          'grant_type' => 'password',
          'client_id' => $client_id,
          'client_secret' => $client_secret,
          'username' => $generate_username,
          'password' => $password,
        ],
      ]);

      $responseBody = $response->getBody()->getContents();
      return json_decode($responseBody, true);
    } catch (ClientException $exception) {
      $statusCode = $exception->getResponse()->getStatusCode();
      if ($statusCode == 401) {
        return new ResourceResponse([
          'status' => 'Error',
          'message' => 'Invalid credentials. Please check your email or password.',
        ], 401);
      }
    }
  }

  public function getToken($user)
  {
    $account = \Drupal\user\Entity\User::load($user->id());

    $username = $account->get('name')[0]->getValue();
    $generate_username = $username['value'];
    $password = base64_encode($user->getEmail());

    $config = $this->configFactory->get('configuration.settings');
    $client_secret = $config->get('client_secret');
    $client_id = $config->get('client_id');
    $public_api_url = $config->get('public_api_url');

    $response = $this->client->post($public_api_url . 'oauth/token', [
      'form_params' => [
        'grant_type' => 'password',
        'client_id' => $client_id,
        'client_secret' => $client_secret,
        'username' => $generate_username,
        'password' => $password,
      ],
    ]);

    $responseBody = $response->getBody()->getContents();
    return json_decode($responseBody, true);
  }

  /** {@inheritdoc} */
  public function permissions(): array
  {
    return [];
  }
}
