<?php

namespace Drupal\user_rest_api\Plugin\rest\resource;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ResourceResponse;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use GuzzleHttp\Client;
use Drupal\Core\Session\AccountInterface;

/**
 * Provides User Login Resource
 *
 * @RestResource (
 *   id = "user_charging_resource",
 *   label = @Translation ("User Charging Resource"),
 *   uri_paths = {
 *    "create" = "api/v1/user-charging"
 *   }
 * )
 */
class UserChargingResource extends ResourceBase {

  /** @var EntityTypeManagerInterface */
  protected EntityTypeManagerInterface $entityTypeManager;

  /** @var Client */
  protected Client $client;

  /** @var ConfigFactoryInterface */
  protected ConfigFactoryInterface $configFactory;

  /** @var AccountInterface */
  protected AccountInterface $currentUser;

  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    array $serializer_formats,
    LoggerInterface $logger,
    EntityTypeManagerInterface $entity_type_manager,
    Client $client,
    ConfigFactoryInterface $config_factory,
    AccountInterface $current_user,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);
    $this->entityTypeManager = $entity_type_manager;
    $this->client = $client;
    $this->configFactory = $config_factory;
    $this->currentUser = $current_user;
  }

  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition,
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->getParameter('serializer.formats'),
      $container->get('logger.factory')->get('sample_rest_resource'),
      $container->get('entity_type.manager'),
      $container->get('http_client'),
      $container->get('config.factory'),
      $container->get('current_user')
    );
  }

  public function post($data): ResourceResponse {
    $user_id = $this->currentUser->id();
    $user = $this->entityTypeManager->getStorage('user')
      ->load($user_id);

    if (!$this->currentUser->isAuthenticated()) {
      return new ResourceResponse([
        'status' => 'Error',
        'message' => 'Unauthorized.',
      ], 401);
    }

    $user_balance = $user->field_customer_balance[0]->value;
    $new_balance = $user_balance - $data['amount'];
    $user->set('field_customer_balance', $new_balance);
    $user->save();

    $this->createCharging($user->id(), $data);

    return new ResourceResponse([
      'status' => 'Success',
      'message' => 'Successful charged.',
    ]);
  }

  public function createCharging($user_id, $data): EntityInterface {
    $current_date = date("Y-m-d");
    $start_time = strtotime($data['start_time']);
    $end_time = strtotime($data['end_time']);
    $charger = $this->getChargerNode($data['charger_id']);
    $node_data = [
      'type' => 'charging_history',
      'title' => 'Charging: ' . $data['charger_id'],
      'field_user' => $user_id,
      'field_charger' => $charger,
      'field_charging_amount' => $data['amount'],
      'field_charging_time' => $data['charging_time'],
      'field_start_time' => $start_time,
      'field_end_time' => $end_time,
      'field_date' => $current_date,
      'field_fee' => $data['fee'],
      'field_kw' => $data['kw'],
    ];

    $node = $this->entityTypeManager->getStorage('node')
      ->create($node_data);

    $node->save();

    return $node;
  }

  public function getChargerNode($chargerId)
  {
    $query = $this->entityTypeManager->getStorage('node')->getQuery()
      ->condition('field_charger_id', $chargerId, '=')
      ->condition('type', 'charger')
      ->accessCheck(TRUE)
      ->range(0, 1); // Limit to one result, assuming you expect only one match.
    $entity_ids = $query->execute();
    return reset($entity_ids);
  }
  /** {@inheritdoc} */
  public function permissions(): array {
    return [];
  }

}
