<?php

namespace Drupal\user_rest_api\Plugin\rest\resource;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ResourceResponse;
use GuzzleHttp\Exception\ClientException;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use GuzzleHttp\Client;

/**
 * Provides User Login Resource
 *
 * @RestResource (
 *   id = "user_login_resource",
 *   label = @Translation ("User Login Resource"),
 *   uri_paths = {
 *    "create" = "api/v1/login"
 *   }
 * )
 */
class UserLoginResource extends ResourceBase {

  /**
   * @var EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * @var Client
   */
  protected Client $client;

  /**
   * @var ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    array $serializer_formats,
    LoggerInterface $logger,
    EntityTypeManagerInterface $entity_type_manager,
    Client $client,
    ConfigFactoryInterface $config_factory
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);
    $this->entityTypeManager = $entity_type_manager;
    $this->client = $client;
    $this->configFactory = $config_factory;
  }

  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition,
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->getParameter('serializer.formats'),
      $container->get('logger.factory')->get('sample_rest_resource'),
      $container->get('entity_type.manager'),
      $container->get('http_client'),
      $container->get('config.factory')
    );
  }

  public function post($data): ResourceResponse {
    $users = $this->entityTypeManager->getStorage('user')
      ->loadByProperties(['mail' => $data['email']]);
    $user = reset($users);

    if (!$user) {
      return new ResourceResponse([
        'status' => 'Error',
        'message' => 'User not found.',
      ], 404);
    }

    $is_social = $user->get('field_is_social')[0]->value;
    if ($is_social == '1') {
      return new ResourceResponse([
        'status' => 'Error',
        'message' => 'Login with Google/Apple account. You are registered with Google/Apple.',
      ], 404);
    }

    try {
      $token = $this->loginUser($user, $data['password']);

      return new ResourceResponse([
        'status' => 'Success',
        'message' => 'Successful user logged in.',
        'data' => [
          'id' => $user->id(),
          'name' => $user->field_customer_name[0]->value,
          'surname' => $user->field_customer_surname[0]->value,
          'email' => $user->mail[0]->value,
          'token_type' => $token['token_type'],
          'expires_in' => $token['expires_in'],
          'access_token' => $token['access_token'],
          'refresh_token' => $token['refresh_token'],
        ],
      ]);
    }
    catch (ClientException $exception) {
      $statusCode = $exception->getResponse()->getStatusCode();
      if ($statusCode) {
        return new ResourceResponse([
          'status' => 'Error',
          'message' => 'Invalid credentials. Please check your email or password.',
        ], $statusCode);
      }
    }
  }

  public function loginUser($user, $password) {
    $account = \Drupal\user\Entity\User::load($user->id());

    $username = $account->get('name')[0]->getValue();
    $generate_username = $username['value'];

    $config = $this->configFactory->get('configuration.settings');
    $client_secret = $config->get('client_secret');
    $client_id = $config->get('client_id');
    $public_api_url = $config->get('public_api_url');

    $response = $this->client->post($public_api_url . 'oauth/token', [
      'form_params' => [
        'grant_type' => 'password',
        'client_id' => $client_id,
        'client_secret' => $client_secret,
        'username' => $generate_username,
        'password' => $password,
      ],
    ]);

    $responseBody = $response->getBody()->getContents();
    return json_decode($responseBody, TRUE);
  }

  /** {@inheritdoc} */
  public function permissions(): array {
    return [];
  }

}
