<?php

namespace Drupal\user_rest_api\Plugin\rest\resource;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ResourceResponse;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use GuzzleHttp\Client;

/**
 * Provides User Resource
 *
 * @RestResource (
 *   id = "user_resource",
 *   label = @Translation ("User Register Resource"),
 *   uri_paths = {
 *    "create" = "api/v1/user/register/customer"
 *   }
 * )
 */
class UserResource extends ResourceBase
{
  /**
   * @var EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * @var Client
   */
  protected Client $client;

  /**
   * @var ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  public function __construct(
    array                      $configuration,
                               $plugin_id,
                               $plugin_definition,
    array                      $serializer_formats,
    LoggerInterface            $logger,
    EntityTypeManagerInterface $entity_type_manager,
    Client                     $client,
    ConfigFactoryInterface     $config_factory
  )
  {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);
    $this->entityTypeManager = $entity_type_manager;
    $this->client = $client;
    $this->configFactory = $config_factory;
  }

  public static function create(
    ContainerInterface $container,
    array              $configuration,
                       $plugin_id,
                       $plugin_definition,
  )
  {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->getParameter('serializer.formats'),
      $container->get('logger.factory')->get('sample_rest_resource'),
      $container->get('entity_type.manager'),
      $container->get('http_client'),
      $container->get('config.factory')
    );
  }

  public function post($data): ResourceResponse
  {
    $user_storage = $this->entityTypeManager->getStorage('user');

    $message_string = "";
    if (empty($data['email'])) {
      $message_string .= "Email. ";
    }
    if (empty($data['name'])) {
      $message_string .= "Name. ";
    }
    if (empty($data['surname'])) {
      $message_string .= "Surname. ";
    }
    if ($message_string) {
      return new ResourceResponse([
        "status" => "Error",
        "message" => "Mandatory Fields Missing",
        "result" => "Required fields: " . $message_string
      ], 400, ['Content-Type' => 'application/json']);
    }

    $existing_user_email = $user_storage->loadByProperties(['mail' => $data['email']]);

    if ($existing_user_email) {
      return new ResourceResponse([
        'status' => 'Failed',
        'message' => 'Email already exists. Please use a different email address.',
      ], 409, ['Content-Type' => 'application/json']);
    }

    $user = $this->setUser($data);
    $token = $this->getToken($user, $data['password']);

    return new ResourceResponse([
      'status' => 'Success',
      'message' => 'Successful user created.',
      'data' => [
        'id' => $user->id(),
        'email' => $data['email'],
        'first_name' => $data['name'],
        'last_name' => $data['surname'],
        'token_type' => $token['token_type'],
        'expires_in' => $token['expires_in'],
        'access_token' => $token['access_token'],
        'refresh_token' => $token['refresh_token']
      ]
    ]);
  }

  private function setUser($data)
  {
    $user = $this->entityTypeManager->getStorage('user')->create();

    $address = [
      'country_code' => $data['billing_address']['country'],
      'address_line1' => $data['billing_address']['address'],
      'locality' => $data['billing_address']['city'],
    ];

    $user->setEmail($data['email']);
    $user->setPassword($data['password']);
    $user->addRole('customer');
    $user->set('preferred_admin_langcode', 'en');
    $user->set('init', $data['email']);
    $user->set('field_customer_name', $data['name']);
    $user->set('field_customer_surname', $data['surname']);
    $user->set('field_customer_billing_address', $address);
    $user->activate();

    $user->save();

    return $user;
  }

  private function getToken($user, $password)
  {
    $account = \Drupal\user\Entity\User::load($user->id());

    $username = $account->get('name')[0]->getValue();
    $generate_username = $username['value'];

    $config = $this->configFactory->get('configuration.settings');
    $client_secret = $config->get('client_secret');
    $client_id = $config->get('client_id');
    $public_api_url = $config->get('public_api_url');

    $response = $this->client->post($public_api_url . 'oauth/token', [
      'form_params' => [
        'grant_type' => 'password',
        'client_id' => $client_id,
        'client_secret' => $client_secret,
        'username' => $generate_username,
        'password' => $password,
      ],
    ]);

    $responseBody = $response->getBody()->getContents();
    return json_decode($responseBody, true);
  }

  /** {@inheritdoc} */
  public function permissions(): array
  {
    return [];
  }
}
