<?php

namespace Drupal\user_rest_api\Plugin\rest\resource;

use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ResourceResponse;
use Symfony\Component\HttpFoundation\Request;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use GuzzleHttp\Client;
use Drupal\Core\Session\AccountInterface;

/**
 * Provides User Update Resource
 *
 * @RestResource (
 *   id = "user_update_resource",
 *   label = @Translation ("User Update Resource"),
 *   uri_paths = {
 *    "create" = "api/v1/user_update"
 *   }
 * )
 */
class UserUpdateResource extends ResourceBase
{
  /**
   * @var EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * @var Client
   */
  protected Client $client;

  /**
   * @var AccountInterface
   */
  protected AccountInterface $currentUser;

  /**
   * @var Request
   */
  protected Request $request;

  public function __construct(
    array                      $configuration,
                               $plugin_id,
                               $plugin_definition,
    array                      $serializer_formats,
    LoggerInterface            $logger,
    EntityTypeManagerInterface $entity_type_manager,
    AccountInterface           $current_user,
    Request                    $request,
  )
  {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);
    $this->entityTypeManager = $entity_type_manager;
    $this->currentUser = $current_user;
    $this->request = $request;
  }

  public static function create(
    ContainerInterface $container,
    array              $configuration,
                       $plugin_id,
                       $plugin_definition,
  )
  {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->getParameter('serializer.formats'),
      $container->get('logger.factory')->get('sample_rest_resource'),
      $container->get('entity_type.manager'),
      $container->get('current_user'),
      $container->get('request_stack')->getCurrentRequest()
    );
  }

  public function post($data): ResourceResponse
  {
    $id = $this->request->query->get('id');
    $user = $this->entityTypeManager->getStorage('user')->load($id);

    if (!$user) {
      return new ResourceResponse([
        'status' => 'Error',
        'message' => 'User not found.',
      ]);
    }

    if ($this->currentUser->id() != $user->id()) {
      return new ResourceResponse([
        'status' => 'Error',
        'message' => 'You are not allowed.',
      ], 403);
    }

    $userData = $this->updateUser($user, $data);

    return new ResourceResponse([
      'status' => 'Success',
      'message' => 'Successful user updated.',
      'data' => [
        'id' => $userData->id(),
        'name' => $userData->field_customer_name[0]->value,
        'surname' => $userData->field_customer_surname[0]->value,
        'username' => $userData->name[0]->value,
        'email' => $userData->mail[0]->value,
        'billing_address' => [
          'address' => $userData->field_customer_billing_address[0]->address_line1,
          'country' => $userData->field_customer_billing_address[0]->country_code,
          'city' => $userData->field_customer_billing_address[0]->locality,
        ],
      ]
    ]);
  }

  private function updateUser($user, $data)
  {
    $is_social = $user->get('field_is_social')[0]->value;

    if (!empty($data['name'])) {
      $user->set('field_customer_name', $data['name']);
    }

    if (!empty($data['surname'])) {
      $user->set('field_customer_surname', $data['surname']);
    }

    if (!empty($data['password']) && $is_social == '0') {
      $user->setPassword($data['password']);
    }

    if (!empty($data['billing_address'])) {
      $address = [
        'country_code' => $data['billing_address']['country'],
        'address_line1' => $data['billing_address']['address'],
        'locality' => $data['billing_address']['city'],
      ];
      $user->set('field_customer_billing_address', $address);
    }

    $user->save();

    return $user;
  }

  /** {@inheritdoc} */
  public function permissions(): array
  {
    return [];
  }
}
